/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// CommunicationSettings.cpp : implementation file
//
#include "stdafx.h"
#include "ui.h"
#include "CommunicationSettings.h"
#include "RegistrySettings.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "comutil.h"
#include "pim/Maincpp.h"
#include "tpcshell.h"  //-- for SHSendBackToFocusWindow
#include "pim/SettingFunctions.h"



// CCommunicationSettings dialog

IMPLEMENT_DYNAMIC(CCommunicationSettings, CDialog)

CCommunicationSettings::CCommunicationSettings(CWnd* pParent /*=NULL*/)
    : CDialog(CCommunicationSettings::IDD, pParent)
{

}

CCommunicationSettings::~CCommunicationSettings()
{
}

void CCommunicationSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_COMM_EDIT_URL, editUrl);
    DDX_Control(pDX, IDC_COMM_EDIT_USER, editUser);
    DDX_Control(pDX, IDC_COMM_EDIT_PASS, editPassword);
}

BOOL CCommunicationSettings::OnInitDialog()
{
    CString s1;
    s1.LoadString(IDS_COMMUNICATION_SETTINGS); SetWindowText(s1); 
    CDialog::OnInitDialog();	

  #if defined(WIN32_PLATFORM_WFSP)
    if (!dlgCommandBar3.Create(this) ||
        !dlgCommandBar3.InsertMenuBar(IDR_MENUBAR_OKCANCEL))
    {
        //AfxMessageBox(_T("Failed to create CommandBar\n"));
        return FALSE;      // failed to create
    } 
    dlgCommandBar3.Show(true);
  #else
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO); 
    dlgCommandBar.hwndParent = this->GetSafeHwnd(); 
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL; 
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0; 
    dlgCommandBar.cBmpImages = 0; 
    if(!SHCreateMenuBar(&dlgCommandBar)){
        AfxMessageBox(_T("Cannot create command bar!")); 
        return FALSE;
    }
  #endif

    editUrl.SetLimitText(EDIT_MAXLENGTH);
    editUser.SetLimitText(EDIT_MAXLENGTH);
    editPassword.SetLimitText(EDIT_MAXLENGTH);

    wchar_t t[128];
    memset(t, 0, 128*sizeof(wchar_t));

    s1.LoadString(IDS_SETTCOMM_SERVURL); SetDlgItemText(IDC_COMM_SERVURL, s1);
    s1.LoadString(IDS_SETTCOMM_USER); SetDlgItemText(IDC_COMM_USER, s1);
    s1.LoadString(IDS_SETTCOMM_PASS); SetDlgItemText(IDC_COMM_PASS, s1);  //!!
    s1.LoadString(IDS_SETTCOMM_DEVID); SetDlgItemText(IDC_SETTCOMM_DEVID, s1);
    s1.LoadString(IDS_SETTCOMM_CURRIP); SetDlgItemText(IDC_SETTCOMM_CURRIP, s1);

    //----- init values with values read from the registry -------
    CommunicationRegSettings regsettings;
    CString s3, s4;

    regsettings.readFromRegistry();

    s3 = regsettings.cs.getAccessConfig().getSyncURL();     SetDlgItemText(IDC_COMM_EDIT_URL, s3);
    s3 = regsettings.cs.getAccessConfig().getUsername();    SetDlgItemText(IDC_COMM_EDIT_USER, s3);
    s3 = regsettings.cs.getAccessConfig().getPassword();    SetDlgItemText(IDC_COMM_EDIT_PASS, s3);
    devId = regsettings.cs.getDeviceConfig().getDevID();       
    
    if (devId == "") {
        #if defined(WIN32_PLATFORM_WFSP)
            getIMEI(t);
        #endif
        #if defined(WIN32_PLATFORM_PSPC)
            GetSerialNumberFromKernelIoControl(t);
        #endif

        devId = t;
    }
    if(devId == ""){
        // device id could not be retrieved: a default device 
        devId = "fwm-windows-mobile";     
    }

    GetDlgItemText(IDC_SETTCOMM_DEVID, s4); s4+=devId;  
    SetDlgItemText(IDC_SETTCOMM_DEVID, s4);
    s4 = ""; 
    s3 = regsettings.cs.getIp();  
    
    GetDlgItemText(IDC_SETTCOMM_CURRIP, s4); s4+=s3;
    SetDlgItemText(IDC_SETTCOMM_CURRIP, s4);
                                                            
    if(checkStartSync() != 0)
    {
        //--another sync is in progress, disable controls
        editUrl.EnableWindow(FALSE); editUser.EnableWindow(FALSE);
        editPassword.EnableWindow(FALSE);
    }
    else
    {
        #ifdef WIN32_PLATFORM_WFSP
         //-- override back key
         LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
	            SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
         ::SendMessage(dlgCommandBar3, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
        #endif
    }

    //-- set a large int as current selection, so the cursor will be at end
    /*
    editUrl.SetSel(LARGE_INT ,LARGE_INT);  
    editUser.SetSel(LARGE_INT ,LARGE_INT); 
    editPassword.SetSel(LARGE_INT ,LARGE_INT); 
    */
    editUrl.SetSel(0);  
    editUser.SetSel(0); 
    editPassword.SetSel(0); 
    
    editUrl.SetFocus();
    return FALSE;
}

BEGIN_MESSAGE_MAP(CCommunicationSettings, CDialog)
#ifdef WIN32_PLATFORM_WFSP
    ON_MESSAGE(WM_HOTKEY,&CCommunicationSettings::OnHotkey)
#endif
END_MESSAGE_MAP()


void CCommunicationSettings::OnOK()
{
    if(checkStartSync() == 0) {
        //-- save values in registry
        CommunicationRegSettings regsettings(false);
        CString s1;
        _bstr_t bst; 

        GetDlgItemText(IDC_COMM_EDIT_URL, s1); 
        if (s1.IsEmpty()) {
            AfxMessageBox(_T("The URL cannot be empty"));
            return;
        } else {
            bst.Assign(s1.AllocSysString());
            regsettings.cs.getAccessConfig().setSyncURL(bst);	
            bst.Detach();
        }
        GetDlgItemText(IDC_COMM_EDIT_USER, s1); 
        if (s1.IsEmpty()) {
            AfxMessageBox(_T("The username cannot be empty"));
            return;
        } else {
             bst.Assign(s1.AllocSysString());
             regsettings.cs.getAccessConfig().setUsername(bst);
             bst.Detach();	
        }
  
         GetDlgItemText(IDC_COMM_EDIT_PASS, s1); 
         bst.Assign(s1.AllocSysString());
         regsettings.cs.getAccessConfig().setPassword(bst);
         bst.Detach();    
        
         bst.Assign(devId.AllocSysString());	
         regsettings.cs.getDeviceConfig().setDevID(bst);  
         bst.Detach();

         if(!checkStartSync())
             regsettings.writeToRegistry();
    }
 CDialog::OnOK();
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CCommunicationSettings::OnHotkey(WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == VK_TBACK)
        SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
    return 0;
}
#endif

