/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// LogViewer.cpp : implementation file
//

#include "stdafx.h"
#include "LogViewer.h"
#include "base/util/utils.h"
#include "HwndFunctions.h"
#include "notify/timed_msgbox.h"

#include <fstream>
using namespace std;
// CLogViewer dialog

#define LINE_NUMBER 15

IMPLEMENT_DYNAMIC(CLogViewer, CDialog)

CLogViewer::CLogViewer(CWnd* pParent /*=NULL*/)
	: CDialog(CLogViewer::IDD, pParent)
{
    filename = NULL;
    isSyncResults = false;
}

CLogViewer::~CLogViewer()
{
    if(filename)
        {delete [] filename; filename = NULL;}
}

void CLogViewer::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_LOGVIEWER_TEXT, editText);
}


BEGIN_MESSAGE_MAP(CLogViewer, CDialog)
    #if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
        ON_WM_SIZE()
    #endif
ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CLogViewer message handlers

BOOL CLogViewer::OnInitDialog()
{
    CString s1; 

    if(isSyncResults)
        s1.LoadString(IDS_SYNCRESULTS);
    else
        s1.LoadString(IDS_LOGVIEWER);
    SetWindowText(s1);

    CDialog::OnInitDialog();
    
   #ifdef WIN32_PLATFORM_WFSP
    if (!dlgCommandBar3.Create(this) ||
        !dlgCommandBar3.InsertMenuBar(IDR_MENUBAR_CLOSE))
    {
        //AfxMessageBox(_T("Failed to create CommandBar\n"));
        return FALSE;      // failed to create
    } 
    dlgCommandBar3.Show(true);
    editText.SetWindowPos(&CWnd::wndBottom,0,0,
        GetDeviceCaps(this->GetDC()->GetSafeHdc(),HORZRES),
        GetDeviceCaps(this->GetDC()->GetSafeHdc(),VERTRES) - 20,
        NULL);
  #else
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO); 
    dlgCommandBar.hwndParent = this->GetSafeHwnd(); 
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_SK1_CLOSE; 
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0; 
    dlgCommandBar.cBmpImages = 0; 
    if(!SHCreateMenuBar(&dlgCommandBar)){
        AfxMessageBox(_T("Cannot create command bar!\n\n")); 
    }
  #endif

    brush = new CBrush(RGB(255,255,255));

    SetDlgItemText(IDC_LOGVIEWER_TEXT,_T(""));
    if(filename == NULL)
    {
        HwndFunctions::closePreviousMsgBox();
        TimedMessageBox(GetSafeHwnd(), _T("Log file not found!"), _T("Funambol alert"), MB_OK | MB_ICONHAND, INFINITE);
        OnOK();
    }
    else
    {
        if(loadFile(filename) == -1)
            OnOK();
    }

    editText.SetSel(0,0);
    editText.SetFocus();
    return FALSE;
}

int CLogViewer::loadFile(const char* _filename)
{
    editText.ResetContent();
    char* s1 = new char[2048];
    int nLines=0;
    strcpy(s1,""); 
    ifstream ifs(_filename);
    unsigned long n=0, bytesRead=0; 
    if(! ifs)
    {
        AfxMessageBox(_T("Log file not found!"), MB_OK | MB_ICONHAND);
        return -1;
    }
    else
    {
        SetCursor(LoadCursor(NULL, IDC_WAIT));
        while ( (!ifs.eof()) && (! ifs.bad()) )
        {
            strcpy(s1,"");
            ifs.getline(s1,2048,'\n');
            //strcat(s1, "\r\n");
            CString temp1(s1);
            /*editText.SetSel(-1,-1);
            editText.ReplaceSel(temp1);*/
            editText.AddString(temp1);
            if(nLines <LINE_NUMBER)
                nLines++;
        }
        ifs.close();
        editText.SetCurSel(1);

        SetCursor(LoadCursor(NULL, IDC_ARROW));
    }
    
    if(nLines < LINE_NUMBER) 
        //&& (nLines >0)){
      editText.ShowScrollBar(FALSE);
       // nLines = -1;
    //};

    editText.SetSel(0,0);
    editText.SetFocus();

    //-- finally
    if(s1)
    {  delete [] s1; s1=NULL;    }
    
    return 0;
}

void CLogViewer::setFile(const char *_filename)
{
    this->filename = stringdup(_filename);
}

void CLogViewer::setIsSyncResults(bool value)                               
{
    isSyncResults = value;
}
#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CLogViewer::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
    int resId;
    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
        resId = IDD_UI_LOGVIEWER_WIDE; break;
    case DRA::Portrait:
        resId = IDD_UI_LOGVIEWER; break;
    case DRA::Square:
        resId = IDD_UI_LOGVIEWER_SQUARE; break;
    default:
        resId = IDD_UI_LOGVIEWER; break;
    };

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
}
#endif

HBRUSH CLogViewer::OnCtlColor( CDC* pDC, CWnd* pWnd, UINT nCtlColor){
   pDC->SetBkColor(RGB(255,255,255));
   return (HBRUSH) (brush->GetSafeHandle());
}
