/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// TaskSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "TaskSettings.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "comutil.h"  //  _bst_t
#include "RegistrySettings.h"
#include "pim/Maincpp.h"
#include "tpcshell.h"  //-- for SHSendBackToFocusWindow

// CTaskSettings dialog

IMPLEMENT_DYNAMIC(CTaskSettings, CDialog)

CTaskSettings::CTaskSettings(CWnd* pParent /*=NULL*/)
	: CDialog(CTaskSettings::IDD, pParent)
{

}

CTaskSettings::~CTaskSettings()
{
}

void CTaskSettings::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
    #if defined(WIN32_PLATFORM_WFSP)
        DDX_Control(pDX, IDC_SETTTASK_SPIN1, spinSyncType );
    #endif
	DDX_Control(pDX, IDC_TASK_SYNCTYPE, lstSyncType );
	DDX_Control(pDX, IDC_SETTTASK_RADIO1, radioBut1);
	DDX_Control(pDX, IDC_SETTTASK_RADIO2, radioBut2);
	DDX_Control(pDX, IDC_SETTTASK_EDIT_REMOTE, editRemote);
}


BEGIN_MESSAGE_MAP(CTaskSettings, CDialog)
#ifdef WIN32_PLATFORM_WFSP
	ON_MESSAGE(WM_HOTKEY,&CTaskSettings::OnHotkey)
#endif
END_MESSAGE_MAP()


// CTaskSettings message handlers

BOOL CTaskSettings::OnInitDialog()
{
	CString s1;

	//-- set window text title
	s1.LoadString(IDS_TASKS_SETTINGS); SetWindowText(s1);

	CDialog::OnInitDialog(); 

	radioBut1.SetCheck(BST_CHECKED); // to be checked if Registry setting isn't found
    editRemote.SetLimitText(EDIT_MAXLENGTH);

  #if defined(WIN32_PLATFORM_WFSP)
    spinSyncType.SetRange(4,0);	spinSyncType.SetBuddy(&lstSyncType);  

	if (!dlgCommandBar3.Create(this) ||
		!dlgCommandBar3.InsertMenuBar(IDR_MENUBAR_OKCANCEL))
	{
		//AfxMessageBox(_T("Failed to create CommandBar\n"));
		return FALSE;      // failed to create
	} 
	dlgCommandBar3.Show(true);
  #else
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO); 
    dlgCommandBar.hwndParent = this->GetSafeHwnd(); 
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL; 
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0; 
    dlgCommandBar.cBmpImages = 0; 
    if(!SHCreateMenuBar(&dlgCommandBar)){
        AfxMessageBox(_T("Cannot create command bar!")); 
        return FALSE;
    }
  #endif

	//------------------- read string resources ---------- //
	s1.LoadString(IDS_SETTCOMM_SYNCTYPE1);	lstSyncType.AddString(s1); 
	s1.LoadString(IDS_SETTCOMM_SYNCTYPE2);	s1+= " Tasks"; lstSyncType.AddString(s1); 
	s1.LoadString(IDS_SETTCOMM_SYNCTYPE3);	s1+= " Tasks";  lstSyncType.AddString(s1);
    s1.LoadString(IDS_SETTCOMM_SYNCTYPE4);	s1+= " Tasks";	lstSyncType.AddString(s1);
    s1.LoadString(IDS_SETTCOMM_SYNCTYPE5);	s1+= " Tasks";	lstSyncType.AddString(s1);

	s1.LoadString(IDS_SETTTASK_FORMAT1); SetDlgItemText(IDC_SETTTASK_RADIO1, s1);
	s1.LoadString(IDS_SETTTASK_FORMAT2);SetDlgItemText(IDC_SETTTASK_RADIO2, s1);
	s1.LoadString(IDS_SYNCTYPE); SetDlgItemText(IDC_SETTTASK_STATIC_SYNCTYPE,s1);
	s1.LoadString(IDS_REMOTENAME); SetDlgItemText(IDC_SETTTASK_STATIC_REMONAME,s1);
	s1.LoadString(IDS_SETTTASK_SYNCFORMAT); SetDlgItemText(IDC_SETTTASK_STATIC_SYNCFORM,s1);
	

	//----- init values with values read from the registry -------
	SyncRegSettings regsettings("task");
	int pos=0;

	s1 = regsettings.cs.ssconfig.getURI(); SetDlgItemText(IDC_SETTTASK_EDIT_REMOTE,s1);

	pos = SynctypeReg2UI(regsettings.cs.ssconfig.getSync()); 
	lstSyncType.SetCurSel(pos);
	if(regsettings.cs.getUseSif() == "1")
	{
		radioBut1.SetCheck(BST_CHECKED);
		radioBut2.SetCheck(BST_UNCHECKED);
	}
	else
	{
		radioBut2.SetCheck(BST_CHECKED);
		radioBut1.SetCheck(BST_UNCHECKED);
	};

    #ifdef WIN32_PLATFORM_WFSP
	    //-- override back key
	    LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
			SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
	    ::SendMessage(dlgCommandBar3, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
    #endif

	//-- set a large int as current selection, so the cursor will be at end
	editRemote.SetSel(0); //editRemote.SetSel(LARGE_INT ,LARGE_INT);  

	lstSyncType.SetFocus();
	return FALSE; //SET focus
}
void CTaskSettings::OnOK()
{
	//-- save values in registry
	SyncRegSettings regsettings("task", false);
	CString s1; int pos=0; 
	_bstr_t bst; 

	GetDlgItemText(IDC_SETTTASK_EDIT_REMOTE, s1); 
	bst.Assign(s1.AllocSysString());
	regsettings.cs.ssconfig.setURI((char*)bst);
    bst.Detach();

	if (radioBut1.GetCheck() == BST_CHECKED)
		 regsettings.cs.setUseSif(CString("1")); // SIF
	else
		regsettings.cs.setUseSif(CString("0")); //  vCal

	pos = lstSyncType.GetCurSel();
    regsettings.cs.ssconfig.setSync(SynctypeUI2Reg(pos));	

    if(!checkStartSync()){
	    regsettings.writeToRegistry("task", false);
    };
	
	CDialog::OnOK();
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CTaskSettings::OnHotkey(WPARAM wParam, LPARAM lParam)
{
	if (HIWORD(lParam) == VK_TBACK)
		SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
	return 0;
}
#endif