/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef INCL_CLIENT_SETTINGS
#define INCL_CLIENT_SETTINGS

#include "base/util/utils.h"
#include "client/DMTClientConfig.h"
#include "client/MailSourceManagementNode.h"
#include "spdm/ManagementNode.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/DMTree.h"

class ClientSettings : public DMTClientConfig {
	
	private:
        // read only. It is for communication settings
	    CString ipaddress;
        
        // client parameters. they are in the root
        CString push;
        CString pushPort;
        CString sms;
        CString polling;
        CString svrNotified;
        CString path;
        CString isPortal;

        // for the sources
        CString useSif; //contact, calendar, task
        CString dir; //briefcase, notes
        CString schedule;

	public:
		ClientSettings(char* application_uri);
		~ClientSettings();
        
        // generic syncSourceConfig. It has the common fields
        SyncSourceConfig ssconfig;
        // Config for the email
        MailSyncSourceConfig mailssconfig;

        /**
         * read all the AccessConfig configuration of the client.
         * there is the Acc, Ext, Conn nodes. 
         * The parameter needs to close remove the internal variables.
         * Use toClose = false when you want to keep the internal variable (as syncMLNode) used
         * to save the configuration
         */

        BOOL readConfig(BOOL toClose = true);
        BOOL saveConfig();
                
        // Extra data that can be used by the client 

        // for communcation form
        void setIp(CString extip);
        CString getIp();
        
        // only for client
        void setPush(CString extpush);
        CString getPush();
		
        void setPushPost(CString extpushPort);
        CString getPushPort();
		
        void setSms(CString extsms);
        CString getSms();
		
        void setPolling(CString extpolling);
        CString getPolling();
        
        void setSvrNotified(CString extsvrNotified);
        CString getSvrNotified();

        void setPath(CString extpath);
        CString getPath();
        
        void setIsPortal(CString extportal);
        bool getIsPortal();

        /*
        * read sync Sources config. 
        * sourceType: the name of the source (contact, calendar, task...)
        * toClose: if the internal ManagementNode must be freed. They don't when read
        * is used before saving the paramenters
        * If sourceType is "all" all the sources with they basic paramenters are read.
        * the basic parameters are the common int the SyncSource class (like "sync", "type"...)
        * the "last" is not replaced.
        */
        BOOL readConfigSources(const char* sourceType, BOOL toClose = true);

        /*
        * saveAll: to save all the sources config. It is used by the form
        * in which there are all the sources whith the list of the sync property.
        * If not specified saveAll parameter, it saves only the source that was loaded
        * and stored into ssconfig or mailssconfig        
        */          
   
        BOOL saveConfigSources(const char* saveAll, bool alsoLast);
        const char* getConfigSourcesParameter(const char* sourceName, const char* parameter);
        BOOL setConfigSourcesParameter(const char* sourceName, const char* parameter, const char* value);

        // Extra Data for the sources
        void setUseSif(CString extuseSif);
        CString getUseSif();

        void setSchedule(CString extSchedule);
        CString getSchedule();

        void setDir(CString extdir);
        CString getDir();
        
        //void setMailSourceConfig(MailSyncSourceConfig& c, ManagementNode* nn);

};

#endif
