/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "pim/CustomSyncClient.h"
#include "event/ManageListener.h"
#include "notify/timed_msgbox.h"
#include "SyncResults.h"
#include "HwndFunctions.h"

#define ASK_SLOW_TIMEOUT_Q1 30
#define ASK_SLOW_TIMEOUT_Q2 30
#define PART_1 TEXT("The server has requested an update of all items in ")
#define PART_2 TEXT(" on your device.  This process may take up to a few minutes depending on the number of items and network bandwidth.  Do you wish to proceed with this update?")
#define PART_3 TEXT("You will be prompted again by 'Server Update Request' message the next time you connect to the server.  If you wish avoid this message and the subsequent update, you may uncheck %s in 'Sync Settings'.")
#define MAIL_INCLUSIVE TEXT("The server has requested an update of all mail items on your device and the recovery of the remainig part of the current email cannot be performed.")


CustomSyncClient::CustomSyncClient()  {
   
}

CustomSyncClient::~CustomSyncClient() {

}

const wchar_t* getFriendlyName(wstring name) {   
        
    if (wcscmp(name.c_str(), TEXT("contact")) == 0)
        return TEXT("Contacts");
    else if (wcscmp(name.c_str(), TEXT("calendar")) == 0)
        return TEXT("Calendar");
    else if (wcscmp(name.c_str(), TEXT("task")) == 0)
        return TEXT("Tasks");
    else if (wcscmp(name.c_str(),TEXT("briefcase")) == 0)
        return TEXT("Files");
    else if (wcscmp(name.c_str(), TEXT("mail")) == 0)
        return TEXT("Mails");
    else if (wcscmp(name.c_str(), TEXT("note")) == 0)
        return TEXT("Notes");
    else
        return TEXT("-");


}


int CustomSyncClient::continueAfterPrepareSync() {
      
    wchar_t line[768], line2[256];
    
    //
    // sourcename are the name of the source that are populated by the server. 
    // So all the sources that have to be synched
    // nameOfSources are the sources populated by the client. they are the ones that
    // are set for a recover sync
    //

    bool addComma = FALSE, addAnd = FALSE;
    int ret = 0, r = 0;    
    vector<wstring> internalNames;   

    for (unsigned int j = 0; j < sourcenames.size(); j++) {
        bool found = false;

        for (unsigned int l = 0; l < namesOfSources.size(); l++) {
            
            wstring newer = namesOfSources[l];
            wstring old = sourcenames[j];
            
            if (wcsstr(newer.c_str(), old.c_str()) != NULL) {     
                found = true; 
                break;
            }            
        }
                
        if (found == false && modes[j] == 201) {
            internalNames.push_back((wstring)sourcenames[j]);                         
        }
        
    }
    
    if (    internalNames.size() == 1 && 
            wcscmp(internalNames[0].c_str(),_T("mail")) == 0 &&
            isMailInclusive) {

        TimedMessageBox(NULL, MAIL_INCLUSIVE, TEXT("Funambol alert"),
                             MB_OK | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_SLOW_TIMEOUT_Q1 * 1000); // default ASK_TIMEOUT = 10
        ret = -20; 
        LOG.info("Sync aborted because a mail slow sync instead of a one-way-server");

    } else if (internalNames.size() == 0) {
        ret = 0;    
    } else {
    
        HwndFunctions::closePreviousMsgBox();

        wcscpy(line, PART_1); 
        wcscpy(line2, TEXT(""));
        addComma = FALSE;
        addAnd = FALSE;
        
        for (unsigned int k = 0; k < internalNames.size(); k++) {
            
            if(wcscmp(internalNames[k].c_str(),_T("mail")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_MAIL_SLOW;

            if(wcscmp(internalNames[k].c_str(),_T("calendar")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_CALENDAR_SLOW;

            if(wcscmp(internalNames[k].c_str(),_T("contact")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_CONTACTS_SLOW;

            if(wcscmp(internalNames[k].c_str(),_T("task")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_TASKS_SLOW;

            if(wcscmp(internalNames[k].c_str(),_T("briefcase")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_BRIEFCASE_SLOW;

            if(wcscmp(internalNames[k].c_str(),_T("note")) == 0)
                SyncResultsLogger.syncTypes |= SYNCRESULTS_NOTES_SLOW;

        
            if (addComma)
                wcscat(line2, TEXT(", "));
            if (addAnd)
                wcscat(line2, TEXT(" and "));

            const wchar_t* s = getFriendlyName(internalNames[k]);
            wcscat(line2, s);
            
            if (k == internalNames.size() - 2) {
                addComma = FALSE; 
                addAnd = TRUE;
            }
            else { 
                addComma = TRUE; 
                addAnd = FALSE;
            }
            
        }
        
        wcscat(line, line2);
        wcscat(line, PART_2);

        HwndFunctions::closePreviousMsgBox();
        r = TimedMessageBox(NULL, line, TEXT("Funambol alert"),
                             MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_SLOW_TIMEOUT_Q1 * 1000); // default ASK_TIMEOUT = 10

        //r = MessageBox (NULL, line, TEXT ("Funambol alert"), MB_SETFOREGROUND | MB_YESNO | MB_ICONQUESTION);

        if (r == IDYES) {
            ret = 0;
        } else {
            
            //wsprintf(line, TEXT("To avoid the same request from the server remove %s in the Sync Settings panel."), line2);        
            wsprintf(line, PART_3, line2);
            HwndFunctions::closePreviousMsgBox();
            int val = TimedMessageBox(NULL, line, TEXT("Funambol alert"),
                             MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, ASK_SLOW_TIMEOUT_Q2 * 1000); // default ASK_TIMEOUT = 10

            //MessageBox (NULL, line, TEXT ("Funambol alert"), MB_SETFOREGROUND | MB_OK | MB_ICONINFORMATION);
            
            ret = -20; 
            LOG.info("Sync aborted by the user to avoid slow-sync: %s", line2);
        }
    }
    
    return ret;
}

       