/*
* Copyright (C) 2003-2006 Funambol
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "pim/WindowsSyncSource.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "pim/POOMClient2Server.h"
#include "pim/POOMServer2Client.h"
#include "spdm/errors.h"
#include "notify/timed_msgbox.h"
#include "events/SyncSourceListenerClient.h"
#include "OutOfMemoryException.h"


#define ASK_REFRESH_PIM_MESSAGE TEXT("All the local %s will be deleted before adding the server items. Are you sure you want to continue?")
#define ASK_REFRESH_PIM_TIMEOUT 10

static int call;
static int cnew;
static int ckey;
static int cupdated;
static int cdeleted;
           
WindowsSyncSource::WindowsSyncSource(const wchar_t* name, SyncSourceConfig* sc) : SyncSource(name, sc) {
    c = NULL;        
    setPath(TEXT("\\"));
    syncItem = NULL;
    itemError = FALSE;
    friendlyName[0] = 0;  // char 64
    newFromServer = 0;
    updatedFromServer = 0;
    deletedFromServer = 0;
}

WindowsSyncSource::~WindowsSyncSource() {
    if (c) 
        delete c; 
    c = NULL;
    if (syncItem)        
        syncItem = NULL;
}

void WindowsSyncSource::setType(SourceType t) {
    type = t;
}

SourceType WindowsSyncSource::getType() {
    return type;
}


void WindowsSyncSource::setPath(const wchar_t* p) {
    if (p == NULL)
        path = wstrdup(TEXT(""));
    else
        path = wstrdup(p);
    
}

wchar_t* WindowsSyncSource::getPath() {
    return path;
}

/*
* Return the first SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* WindowsSyncSource::getFirstItem() {
    
    if (syncItem) {        
        syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsPOOM(c, getType(), getPath()); // olFolderContact = 10
    
    LOG.info("The client number of %s to sync are %i", friendlyName, c->getAllItemsSize());
    
    int params[6] = {0,0,0,0,0,0}; 
    params[0] = c->getAllItemsSize(); 
    //SyncResultsLogger.updatePim(friendlyName, params, true);
    if(strcmp(config.getSync(),"refresh-client") == 0){               
        SyncResultsLogger.updatePim(friendlyName, params, false);
    }

    if(HwndFunctions::wnd)
        SendMessage(HwndFunctions::wnd, (WM_APP+8), (WPARAM) -1,(LPARAM)c->getAllItemsSize());


    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    call = 0;
    
    ArrayList* list = c->getAllItems(); 
    syncItem = (SyncItem*)((SyncItem*)list->get(call))->clone();       
    fillSyncItem(syncItem, getType());
    return syncItem;
}

/*
* Return the next SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* WindowsSyncSource::getNextItem() {        
    
    if (syncItem) {
        syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    call++;
    if (call == c->getAllItemsSize()) {
        return NULL;
    }     
    syncItem = (SyncItem*)((SyncItem*)c->getAllItems()->get(call))->clone();  
    fillSyncItem(syncItem, getType());
    return syncItem;   
}

/*
* This method is always called by the sync manager. So the setModifiedItemsPOOM is always called
*/

SyncItem* WindowsSyncSource::getFirstNewItem() {
    
    if (syncItem) {
        syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setModifiedItemsPOOM(c, getType(), getPath()); // olFolderContact = 10

    //Log the number of item to sync from client
    LOG.info("The client number of new %s to sync are %i", friendlyName, c->getNewItemsSize());
    LOG.info("The client number of updated %s to sync are %i", friendlyName, c->getUpdatedItemsSize());
    LOG.info("The client number of deleted %s to sync are %i", friendlyName, c->getDeletedItemsSize());

    int params[6]; 
    params[0] = c->getNewItemsSize(); params[1] = c->getUpdatedItemsSize();
    params[2] = c->getDeletedItemsSize(); params[3] = 0;
    params[4] = 0; params[5] = 0;

    //-- 
    SyncResultsLogger.updatePim(friendlyName, params, true);
    
    if (c->getNewItemsSize() == 0) {
        return NULL;
    }    
    cnew = 0;
    
    ArrayList* list = c->getNewItems();
    return (SyncItem*)list->get(cnew)->clone();    
    
}

SyncItem* WindowsSyncSource::getFirstItemKey() {
          
    int deleted = deleteAllItems(getType());
    // it is a replace local so update the params containing the deleted
    int params[6] = {0,0,0,0,0,0};
    params[5] += deletedFromServer; 
    SyncResultsLogger.updatePim(friendlyName, params, false);
    
    // reset the deleted item number. 
    deletedFromServer = 0;

    if (c) { delete c; c = NULL; }
    return NULL; 
}

SyncItem* WindowsSyncSource::getNextItemKey() {
    return NULL;   
}

SyncItem* WindowsSyncSource::getNextNewItem() {    
    
    if (syncItem) {
        //delete syncItem; syncItem = NULL;
        syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    cnew++;
    if (cnew == c->getNewItemsSize()) {
        return NULL;
    }
    
    return (SyncItem*)c->getNewItems()->get(cnew)->clone();    
}

SyncItem* WindowsSyncSource::getFirstUpdatedItem() {
    
    if (syncItem) {
        syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    if (c->getUpdatedItemsSize() == 0) {
        return NULL;
    }    
    cupdated = 0;    

    ArrayList* list = c->getUpdatedItems();    
    return (SyncItem*)list->get(cupdated)->clone();          
    
}

SyncItem* WindowsSyncSource::getNextUpdatedItem() {    
   
    if (syncItem) {        
        syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    cupdated++;
    if (cupdated == c->getUpdatedItemsSize()) {
        return NULL;
    }        
   
    return (SyncItem*)c->getUpdatedItems()->get(cupdated)->clone();         
}

SyncItem* WindowsSyncSource::getFirstDeletedItem() {
   
    if (c == NULL) {
        c = new Container();
    }

    if (c->getDeletedItemsSize() == 0) {
        return NULL;
    }    
    cdeleted = 0;
    ArrayList* list = c->getDeletedItems();

    return (SyncItem*)list->get(cdeleted)->clone();      
}

SyncItem* WindowsSyncSource::getNextDeletedItem() {    
    if (c == NULL)
        return NULL;

    cdeleted++;
    if (cdeleted == c->getDeletedItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getDeletedItems()->get(cdeleted)->clone();    
}

void WindowsSyncSource::setItemStatus(const wchar_t* key, int status) {    
    LOG.debug("key: %ls, status: %i", key, status);     
    if (status > 299 && status != 418 && status != 419)
        itemError = TRUE;
    else
        itemError = FALSE;
}

int WindowsSyncSource::addItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long oid = 0;
    if (!isMemoryAvailable()) {
        LOG.error(LOW_DEVICE_MEMORY);
        throw OutOfMemoryException(0, -50);  //defined in startsync //return ret;
    }

    long h = manageNewItems(&item, getType(), &oid, getPath());
    
    wchar_t tmp[256];
    if (h == 0) {        
        ret = STC_ITEM_ADDED;
        LOG.debug("Added item: %ls LUID: %i", item.getKey(), oid);        
        wsprintf(tmp, TEXT("%i"), oid);
        item.setKey(tmp);  

        newFromServer++;
    }
    return ret;
}

int WindowsSyncSource::updateItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    
    if (!isMemoryAvailable()) {
        LOG.error(LOW_DEVICE_MEMORY);
        throw OutOfMemoryException(0, -50);    //defined in startsync //return ret;
    }

    long h = manageUpdatedItems(&item, getType(), getPath());
    
    if (h == 0) {
        ret = STC_OK;        
        LOG.debug("updated item: %ls", item.getKey());  

        updatedFromServer++;
    }
    return ret;
}

int WindowsSyncSource::deleteItem(SyncItem& item) {
    int ret = STC_COMMAND_FAILED; // command Failed;
    long h = manageDeletedItems(&item, getType());
    
    if (h == 0) {
        ret = STC_OK;        
        LOG.debug("deleted item: %ls", item.getKey());   
        
        deletedFromServer++;
    }
    return ret;    
}

int WindowsSyncSource::beginSync() {
    /*
    OL_CONTACTS     = 10,
    OL_CALENDAR     = 9,
    OL_TASK         = 13,
    OL_BRIEFCASE    = 600,
    OL_NOTES        = 601,
    OL_FAVORITES    = 602
    */
    if (getType() == OL_CONTACTS)
    {
        strcpy(friendlyName, "contacts");
        SyncResultsLogger.setCurrentSource(CURRENT_SOURCE_CONTACT);
    }
    else if (getType() == OL_CALENDAR)
    {
        strcpy(friendlyName, "calendars");
        SyncResultsLogger.setCurrentSource(CURRENT_SOURCE_CALENDAR);
        normalizeAllExceptions();
    }
    else if (getType() == OL_TASK)
    {
        strcpy(friendlyName, "tasks");    
        SyncResultsLogger.setCurrentSource(CURRENT_SOURCE_TASK);
    }
    
    if(HwndFunctions::wnd)
        SendMessage(HwndFunctions::wnd, (WM_APP+6), NULL,(LPARAM) SyncResultsLogger.getCurrentSource());

    // counter to get the number of items from server
    newFromServer = 0;
    updatedFromServer = 0;
    deletedFromServer = 0;
  
    int params[6] = {0,0,0,0,0,0};    
    //-- send 'false' so it doesn't send msg to UI
    SyncResultsLogger.updatePim(friendlyName, params, false);

    return 0;    
}

int WindowsSyncSource::endSync() {
      
    writeCurrentItems(getType(), getPath());
    if (TRUE == itemError) {
        return ERR_ITEM_ERROR;
    }
    else {
        //Log the number of item synced from server
        LOG.info("The server number of new %s synced are %i", friendlyName, newFromServer);
        LOG.info("The server number of updated %s synced are %i", friendlyName, updatedFromServer);
        LOG.info("The server number of deleted %s synced are %i", friendlyName, deletedFromServer);
      
        int params[6];
        params[3] = newFromServer; params[4] = updatedFromServer;
        params[5] = deletedFromServer; params[0] = 0;
        params[1] = 0; params[2] = 0;
        SyncResultsLogger.updatePim(friendlyName, params,false);
        return 0;
    }
}

void WindowsSyncSource::assign(WindowsSyncSource& s) {
    setSyncMode(s.getSyncMode());
    setLastSync(s.getLastSync());
    setNextSync(s.getNextSync());    
    setLastAnchor(s.getLastAnchor());
    setNextAnchor(s.getNextAnchor());   
    setFilter(s.getFilter());
    setPath(getPath());
    setType(getType());
    setReport(s.getReport());
}